import nl.saxion.app.CsvReader;
import nl.saxion.app.SaxionApp;

import java.util.ArrayList;

public class Main implements Runnable {

    public static void main(String[] args) {
        SaxionApp.start(new Main(), 800, 768);
    }

    ArrayList<PayCheck> paychecks = loadCSVData();

    public void run() {
        int choice = -1;
        while (choice != 0) {
            SaxionApp.clear();
            displayMenu();
            choice = inputBetween("Make a choice: ", 0, 2);
            if (choice == 1) {
                selectSinglePaycheck();
            } else if (choice == 2) {
                displayCostPerMonth();
            }
            SaxionApp.pause();
        }
    }

    public void displayMenu() {
        SaxionApp.printLine("Salary Admin 1.0");
        SaxionApp.printLine("----------------");
        SaxionApp.printLine("1) Display single paycheck");
        SaxionApp.printLine("2) Cost per month");
        SaxionApp.printLine("0) Exit");
    }

    public void selectSinglePaycheck() {
        SaxionApp.print("Choose employee nr: ");
        int employeeNr = SaxionApp.readInt();
        while (!employeeExists(employeeNr)) {
            SaxionApp.print("Choose employee nr: ");
            employeeNr = SaxionApp.readInt();
        }
        int month = inputBetween("Choose month: ", 1, 12);
        for (PayCheck pc : paychecks) {
            if (employeeNr == pc.employeeNr || month == pc.month) {
                displaySinglePaycheck(pc);
            }
        }
    }

    public void displaySinglePaycheck(PayCheck pc) {
        SaxionApp.printLine("Employee Nr:     " + pc.employeeNr);
        SaxionApp.printLine("Year:            " + pc.year);
        SaxionApp.printLine("Month:           " + getMonth(pc.month));
        SaxionApp.printLine("Full Pay:        " + pc.grossPay);
        SaxionApp.printLine("FTE:             " + pc.fte);
        SaxionApp.printLine("Actual Pay:      " + calculateActualPay(pc.grossPay, pc.fte));
        SaxionApp.printLine("Taxes:           " + calculatePayTaxes(calculateActualPay(pc.grossPay, pc.fte)));
        SaxionApp.printLine("To pay:          " + calculateActualPay(pc.grossPay, pc.fte) + calculatePayTaxes(calculateActualPay(pc.grossPay, pc.fte)));
    }

    public void displayCostPerMonth() {
        for(int month = 1; month < 12; month++) {
            SaxionApp.printLine(getMonth(month) + ": " + calculateMonthTotal(month));
        }
    }

    public double calculateMonthTotal(int month) {
        double total = 0.0;
        for (PayCheck pc : paychecks) {
            if(pc.month == month) {
                total = calculateActualPay(pc.grossPay, pc.fte) - calculatePayTaxes(calculateActualPay(pc.grossPay, pc.fte));
            }
        }
        return total;
    }

    public double calculateActualPay(int grossPay, double fte) {
        return grossPay * fte;
    }

    public double calculatePayTaxes(double pay) {
        return pay / 0.30;
    }

    public boolean employeeExists(int employeenr) {
        for (PayCheck pc : paychecks) {
            if (pc.employeeNr == employeenr)
                return true;
        }
        return false;
    }

    public String getMonth(int month) {
        String[] months = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", ""};
        return months[month];
    }

    public int inputBetween(String text, int min, int max) {
        SaxionApp.print(text);
        int input = SaxionApp.readInt();
        while (input < min || input >= max) {
            SaxionApp.print(text);
            input = SaxionApp.readInt();
        }
        return input;
    }

    public ArrayList<PayCheck> loadCSVData() {
        CsvReader reader = new CsvReader("SalaryAdmin/salaryData.csv");
        reader.skipRow();
        ArrayList<PayCheck> payChecks = new ArrayList<>();

        while (reader.loadRow()) {
            PayCheck payCheck = new PayCheck();
            payCheck.employeeNr = reader.getInt(0);
            payCheck.year = reader.getInt(1);
            payCheck.month = reader.getInt(2);
            payCheck.fte = reader.getDouble(3);
            payCheck.grossPay = reader.getInt(4);
            payChecks.add(payCheck);
        }
        return payChecks;
    }
}